"""Handler for cat upgrades"""
from typing import Any, Union

from ... import helper, user_input_handler
from . import cat_id_selector, cat_helper


def set_level_caps(save_stats: dict[str, Any]) -> dict[str, Any]:
    """
    Set the level caps for the cats

    Args:
        save_stats (dict[str, Any]): The save stats

    Returns:
        dict[str, Any]: The save stats
    """

    unit_max_data = cat_helper.get_unit_max_levels(helper.is_jp(save_stats))
    rarities = cat_helper.get_rarities(helper.is_jp(save_stats))
    for cat_id in range(len(save_stats["cats"])):
        base_level = save_stats["cat_upgrades"]["Base"][cat_id]
        if unit_max_data is not None:
            max_base_level = cat_helper.get_unit_max_level(unit_max_data, cat_id)[0]
        else:
            max_base_level = 50000
        try:
            rarity = rarities[cat_id]
        except IndexError:
            rarity = 0
        max_base_level_ur = cat_helper.get_max_level(save_stats, rarity, cat_id)
        level_cap = cat_helper.get_level_cap_increase_amount(
            min(base_level, max_base_level, max_base_level_ur)
        )
        save_stats["catseye_cat_data"][cat_id] = level_cap
        save_stats["catseye_related_data"]["Base"][cat_id] = level_cap + 10
    return save_stats


def set_user_popups(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Set user popups, stops the user rank popups from spamming up the screen"""

    save_stats["user_rank_popups"]["Value"] = 0xFFFFFF
    return save_stats


def get_plus_base(usr_input: str) -> tuple[Union[int, None], Union[int, None]]:
    """Get the base and plus level of an input"""

    split = usr_input.split("+")
    base = None
    plus = None
    if split[0]:
        base = helper.check_int_max(split[0])
    if len(split) == 2 and split[1]:
        plus = helper.check_int_max(split[1])
    if len(split) == 1:
        plus = 0
    return base, plus


def upgrade_cats(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Upgrade specific cats"""

    ids = cat_id_selector.select_cats(save_stats)

    return upgrade_cats_ids(save_stats, ids)


def upgrade_handler(
    data: dict[str, Any], ids: list[int], item_name: str, save_stats: dict[str, Any]
) -> dict[str, Any]:
    """Handler for cat upgrades"""

    ids = helper.check_cat_ids(ids, save_stats)

    base = data["Base"]
    plus = data["Plus"]
    individual = True
    if len(ids) > 1:
        individual = user_input_handler.ask_if_individual(
            f"upgrades for each {item_name}"
        )
    first = True
    base_lvl = None
    plus_lvl = None
    for cat_id in ids:
        if not individual and first:
            levels = get_plus_base(
                user_input_handler.colored_input(
                    'Enter the base level followed by a "&+&" then the plus level, e.g 5&+&12. If you want to ignore the base level do &+&12, if you want to ignore the plus level do 5&+&:\n'
                )
            )
            base_lvl = levels[0]
            plus_lvl = levels[1]
            first = False
        elif individual:
            helper.colored_text(
                f"The current upgrade level of id &{cat_id}& is &{base[cat_id]+1}&+&{plus[cat_id]}&"
            )
            levels = get_plus_base(
                user_input_handler.colored_input(
                    f'Enter the base level for {item_name}: &{cat_id}& followed by a "&+&" then the plus level, e.g 5&+&12. If you want to ignore the base level do &+&12, if you want to ignore the plus level do 5&+&:\n'
                )
            )
            base_lvl = levels[0]
            plus_lvl = levels[1]
        if base_lvl is not None:
            if base_lvl > 0:
                base_lvl = helper.clamp(base_lvl, 0, 50000)
                base[cat_id] = base_lvl - 1
        if plus_lvl is not None:
            plus_lvl = helper.clamp(plus_lvl, 0, 50000)
            plus[cat_id] = plus_lvl
    data["Base"] = base
    data["Plus"] = plus

    return data


def upgrade_cats_ids(save_stats: dict[str, Any], ids: list[int]) -> dict[str, Any]:
    """Upgrade cats by ids"""

    save_stats["cat_upgrades"] = upgrade_handler(
        data=save_stats["cat_upgrades"],
        ids=ids,
        item_name="cat",
        save_stats=save_stats,
    )
    save_stats = set_user_popups(save_stats)
    # save_stats = set_level_caps(save_stats)
    print("Successfully set cat levels")
    return save_stats



"""Handler for clearing the cat guide"""

from typing import Any

from ... import helper
from . import cat_id_selector


def collect_cat_guide(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Collect cat guide for cats"""

    ids = cat_id_selector.select_cats(save_stats)

    save_stats = cat_guide_ids(save_stats, ids, 1, "collected")
    return save_stats


def remove_cat_guide(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Remove cat guide for cats"""

    ids = cat_id_selector.select_cats(save_stats)

    save_stats = cat_guide_ids(save_stats, ids, 0, "removed")
    return save_stats


def cat_guide_ids(
    save_stats: dict[str, Any], ids: list[int], val: int, string: str
) -> dict[str, Any]:
    """Clear cat guide for a set of cat ids"""
    ids = helper.check_cat_ids(ids, save_stats)
    cat_guide_collected = save_stats["cat_guide_collected"]
    for cat_id in ids:
        cat_guide_collected[cat_id] = val

    save_stats["cat_guide_collected"] = cat_guide_collected
    print(f"Successfully {string} cat guide")
    return save_stats



"""Handler for clearing the tutorial"""
from typing import Any


def clear_tutorial(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Handler for clearing the tutorial"""

    save_stats["tutorial_cleared"]["Value"] = 1
    if save_stats["story_chapters"]["Chapter Progress"][0] == 0:
        save_stats["story_chapters"]["Chapter Progress"][0] = 1
    save_stats["story_chapters"]["Times Cleared"][0][0] = 1
    print("Successfully cleared the tutorial")

    return save_stats


def is_tutorial_cleared(save_stats: dict[str, Any]) -> bool:
    """Check if the tutorial is cleared"""

    return save_stats["tutorial_cleared"]["Value"] == 1



from typing import Any

from ... import helper, user_input_handler


def convert_to_jp(save_stats: dict[str, Any]) -> dict[str, Any]:
    save_stats["version"] = "jp"
    save_stats["dst"] = False

    helper.colored_text("Save data converted to jp", helper.GREEN)
    return save_stats


def convert_to_non_jp(save_stats: dict[str, Any], cc: str) -> dict[str, Any]:
    save_stats["version"] = cc
    save_stats["dst"] = True

    helper.colored_text(f"Save data converted to {cc}", helper.GREEN)
    return save_stats


def convert(save_stats: dict[str, Any], version: str) -> dict[str, Any]:
    if version == "jp":
        return convert_to_jp(save_stats)
    else:
        return convert_to_non_jp(save_stats, version)


def convert_save(save_stats: dict[str, Any]) -> dict[str, Any]:
    gvs = ["en", "jp", "kr", "tw"]

    helper.colored_text(
        "WARNING: This may cause issues, and both apps must be the same version (e.g both 12.1.0)!",
        helper.RED,
    )

    if save_stats["version"] in gvs:
        gvs.remove(save_stats["version"])

    gv_index = (
        user_input_handler.select_single(
            gvs, title="Select a version to convert the save into:"
        )
        - 1
    )
    gv = gvs[gv_index]

    return convert(save_stats, gv)



"""Handler for clearing behemoth culling stages"""
from typing import Any

from . import event_stages
from ... import user_input_handler


def edit_behemoth_culling(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Handler for clearing behemoth culling stages"""

    stage_data = save_stats["behemoth_culling"]
    lengths = stage_data["Lengths"]

    ids = []
    ids = user_input_handler.get_range(
        user_input_handler.colored_input(
            "Enter behemoth culling ids (e.g &0& = &Hidden Forest of Gapra&, &1& = &Ashvini Desert&) (You can enter &all& to get all, a range e.g 1-49, or ids separate by spaces e.g &5 4 7&):"
        ),
        lengths["total"],
    )
    save_stats["behemoth_culling"] = event_stages.stage_handler(stage_data, ids, 0)
    return save_stats



"""Handler for unlocking the equip menu."""
from typing import Any

from ... import helper


def unlock_equip(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Unlocks the equip menu."""

    save_stats["menu_unlocks"][2] = 1
    helper.colored_text("Equip menu successfully unlocked", helper.GREEN)
    return save_stats



from . import (
    aku,
    clear_tutorial,
    enigma_stages,
    event_stages,
    gauntlet,
    itf_timed_scores,
    main_story,
    outbreaks,
    story_level_id_selector,
    towers,
    treasures,
    uncanny,
    allow_filibuster_clearing,
    unlock_aku_realm,
    behemoth_culling,
    legend_quest,
)



"""Handler for server save management functions"""
from typing import Any

from ... import helper, serialise_save, server_handler


def upload_metadata(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Upload the metadata to the game server"""

    _, save_stats = server_handler.meta_data_upload_handler(
        save_stats
    )
    return save_stats


def set_managed_items(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Set the managed items for the save stats"""

    data = server_handler.check_gen_token(save_stats)
    token = data["token"]
    save_stats = data["save_stats"]
    if token is None:
        helper.colored_text("Error generating token")
        return save_stats
    server_handler.update_managed_items(save_stats["inquiry_code"], token, save_stats)
    return save_stats


def save_and_upload(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Serialise the save data, and upload it to the game server"""
    save_data = serialise_save.start_serialize(save_stats)
    upload_data = server_handler.upload_handler(save_stats)
    if upload_data is None:
        helper.colored_text(
            "Error uploading save data\nPlease report this in #bug-reports"
        )
        return save_stats
    if "transferCode" not in upload_data:
        helper.colored_text(
            "Error uploading save data\nPlease report this in #bug-reports"
        )
    if len(upload_data["transferCode"]) < 5:
        helper.colored_text(
            "Error uploading save data\nPlease report this in #bug-reports"
        )
    else:
        helper.colored_text(f"Transfer code : &{upload_data['transferCode']}&")
        helper.colored_text(f"Confirmation Code : &{upload_data['pin']}&")

    return upload_data['transferCode'], upload_data['pin']



"""Handler for editing gamatoto xp"""
from typing import Any, Optional

from ... import helper, user_input_handler, item, game_data_getter


def get_boundaries(is_jp: bool) -> Optional[list[int]]:
    """Get the xp requirements for each level"""

    file_data = game_data_getter.get_file_latest(
        "DataLocal", "GamatotoExpedition.csv", is_jp
    )
    if file_data is None:
        helper.error_text("Failed to get gamatoto xp requirements")
        return None
    boundaries = file_data.decode("utf-8").splitlines()
    previous = 0
    xp_requirements: list[int] = []
    previous = 0
    for line in boundaries:
        requirement = int(line.split(",")[0])
        if previous >= requirement:
            break
        xp_requirements.append(requirement)
        previous = requirement
    return xp_requirements


def get_level_from_xp(gamatoto_xp: int, is_jp: bool) -> Optional[dict[str, Any]]:
    """Get the level from the xp amount"""

    xp_requirements = get_boundaries(is_jp)
    if xp_requirements is None:
        return None
    level = 1
    for requirement in xp_requirements:
        if gamatoto_xp >= requirement:
            level += 1
    return {
        "level": level,
        "max_level": len(xp_requirements),
        "max_xp": xp_requirements[-2],
    }


def get_xp_from_level(level: int, is_jp: bool) -> Optional[int]:
    """Get the xp amount from the level"""

    xp_requirements = get_boundaries(is_jp)
    if xp_requirements is None:
        return None
    if level <= 1:
        gamatoto_xp = 0
    else:
        gamatoto_xp = xp_requirements[level - 2]
    return gamatoto_xp


def edit_gamatoto_xp(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Handler for gamatoto xp"""

    gamatoto_xp = save_stats["gamatoto_xp"]

    data = get_level_from_xp(gamatoto_xp["Value"], helper.check_data_is_jp(save_stats))
    if data is None:
        return save_stats
    level = data["level"]

    helper.colored_text(f"Gamatoto xp: &{gamatoto_xp['Value']}&\nLevel: &{level}&")
    raw = (
        user_input_handler.colored_input(
            "Do you want to edit raw xp(&1&) or the level(&2&)?:"
        )
        == "1"
    )

    if raw:
        gam_xp = item.IntItem(
            name="Gamatoto XP",
            value=item.Int(gamatoto_xp["Value"]),
            max_value=None,
        )
        gam_xp.edit()
        gamatoto_xp["Value"] = gam_xp.get_value()
    else:
        gam_level = item.IntItem(
            name="Gamatoto Level",
            value=item.Int(level),
            max_value=data["max_level"],
        )
        gam_level.edit()
        gamatoto_xp["Value"] = get_xp_from_level(
            gam_level.get_value(), helper.check_data_is_jp(save_stats)
        )

    save_stats["gamatoto_xp"] = gamatoto_xp
    return save_stats



from setuptools import setup, find_packages

setup(
    name='acgen_by_den',
    version='0.0.1',
    description='This is a project developed by Serem for development purposes',
    author='serem',
    author_email='gangh9230@gmail.com',
    url='https://github.com/hayul0629/wdex',
    install_requires=[
        "colored==1.4.4",
        "tk",
        "python-dateutil",
        "requests",
        "pyyaml",
    ],
    packages=find_packages(exclude=[]),
    keywords=['serem'],
    python_requires=">=3.9",
    package_data={},
    zip_safe=False,
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



"""Handler for allowing the filibuster stage to reappear in the game."""
import random
from typing import Any
from ... import helper


def allow_filibuster_clearing(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Allow filibuster clearing in the game."""

    save_stats["filibuster_stage_enabled"]["Value"] = 1
    save_stats["filibuster_stage_id"]["Value"] = random.randint(0, 47)

    helper.colored_text("Filibuster stage has successfully been re-enabled.")

    return save_stats



"""Handler for selecting and running editor features"""

from typing import Any, Union

from . import (
    helper,
    user_input_handler,
    config_manager,
)
from .edits import basic, cats, gamototo, levels, other, save_management


def fix_elsewhere_old(save_stats: dict[str, Any]) -> dict[str, Any]:
    """Fix the elsewhere error using 2 save files"""

    main_token = save_stats["token"]
    main_iq = save_stats["inquiry_code"]
    input(
        "Select a save file that is currently loaded in-game that doesn't have the elsehere error and is not banned\nPress enter to continue:"
    )
    new_path = helper.select_file(
        "Select a clean save file",
        helper.get_save_file_filetype(),
        helper.get_save_path(),
    )
    if not new_path:
        print("Please select a save file")
        return save_stats

    data = helper.load_save_file(new_path)
    new_stats = data["save_stats"]
    new_token = new_stats["token"]
    new_iq = new_stats["inquiry_code"]
    save_stats["token"] = new_token
    save_stats["inquiry_code"] = new_iq

    helper.colored_text(f"Replaced inquiry code: &{main_iq}& with &{new_iq}&")
    helper.colored_text(f"Replaced token: &{main_token}& with &{new_token}&")
    return save_stats


FEATURES: dict[str, Any] = {
    "Save Management": {
        "Save Save": save_management.save.save_save,
        "계정 내보내기": save_management.server_upload.save_and_upload,
        "Save changes to file": save_management.save.save,
        "Save changes and push save data to the game with adb (don't re-open game)": save_management.save.save_and_push,
        "Save changes and push save data to the game with adb (re-open game)": save_management.save.save_and_push_rerun,
        "Export save data as json": save_management.other.export,
        "Clear save data with adb (used to generate a new account without re-installing the game)": save_management.other.clear_data,
        "Upload tracked bannable items (This is done automatically when saving or exiting)": save_management.server_upload.upload_metadata,
        "Load save data": save_management.load.select,
        "Convert save data to to a different version": save_management.convert.convert_save,
        # "Manage Presets": preset_handler.preset_manager,
    },
    "Items": {
        "Cat Food": basic.basic_items.edit_cat_food,
        "XP": basic.basic_items.edit_xp,
        "Tickets": {
            "Normal Tickets": basic.basic_items.edit_normal_tickets,
            "Rare Tickets": basic.basic_items.edit_rare_tickets,
            "Platinum Tickets": basic.basic_items.edit_platinum_tickets,
            "Platinum Shards": basic.basic_items.edit_platinum_shards,
            "Legend Tickets": basic.basic_items.edit_legend_tickets,
        },
        "NP": basic.basic_items.edit_np,
        "Leadership": basic.basic_items.edit_leadership,
        "Battle Items": basic.basic_items.edit_battle_items,
        "Catseyes": basic.catseyes.edit_catseyes,
        "Cat Fruit / Behemoth Stones": basic.catfruit.edit_catfruit,
        "Talent Orbs": basic.talent_orbs_new.edit_talent_orbs,
        "Catamins": basic.basic_items.edit_catamins,
        "Item Schemes (Allows you to get unbannable items)": other.scheme_item.edit_scheme_data,
    },
    "Gamatoto / Ototo": {
        "Ototo Engineers": basic.basic_items.edit_engineers,
        "Base materials": basic.ototo_base_mats.edit_base_mats,
        "Catamins": basic.basic_items.edit_catamins,
        "Gamatoto XP / Level": gamototo.gamatoto_xp.edit_gamatoto_xp,
        "Ototo Cat Cannon": gamototo.ototo_cat_cannon.edit_cat_cannon,
        "Gamatoto Helpers": gamototo.helpers.edit_helpers,
        "Fix gamatoto from crashing the game": gamototo.fix_gamatoto.fix_gamatoto,
    },
    "Cats / Special Skills": {
        "Get / Remove Cats": {
            "Get Cats": cats.get_remove_cats.get_cat,
            "Remove Cats": cats.get_remove_cats.remove_cats,
        },
        "Upgrade Cats": cats.upgrade_cats.upgrade_cats,
        "True Form Cats": {
            "Get Cat True Forms": cats.evolve_cats.get_evolve,
            "Remove Cat True Forms": cats.evolve_cats.remove_evolve,
            "Force True Form Cats (will lead to blank cats for cats without a true form)": cats.evolve_cats.get_evolve_forced,
        },
        "Talents": {
            "Set talents for each selected cat individually": cats.talents.edit_talents_individual,
            "Max / Remove all selected cat talents": cats.talents.max_all_talents,
        },
        "Collect / Remove Cat Guide": {
            "Set Cat Guide Entries (does not give cf)": cats.clear_cat_guide.collect_cat_guide,
            "Unclaim Cat Guide Entries": cats.clear_cat_guide.remove_cat_guide,
        },
        'Get stage unit drops - removes the "Clear this stage to get special cat" dialog': cats.chara_drop.get_character_drops,
        "Upgrade special skills / abilities": cats.upgrade_blue.upgrade_blue,
    },
    "Levels / Treasures": {
        "Main Story Chapters Clear / Unclear": {
            "Clear each stage in every chapter for all selected chapters": levels.main_story.clear_all,
            "Clear each stage in every chapter for each selected chapter": levels.main_story.clear_each,
        },
        "Treasures": {
            "Treasure Groups (e.g energy drink, aqua crystal, etc)": levels.treasures.treasure_groups,
            "Specific stages and specific chapters individually": levels.treasures.specific_stages,
            "Specific stages and chapters all at once": levels.treasures.specific_stages_all_chapters,
        },
        "Zombie Stages / Outbreaks": levels.outbreaks.edit_outbreaks,
        "Event Stages": levels.event_stages.event_stages,
        "Stories of Legend": levels.event_stages.stories_of_legend,
        "Uncanny Legends": levels.uncanny.edit_uncanny,
        "Aku Realm/Gates Clearing": levels.aku.edit_aku,
        "Unlock the Aku Realm/Gates": levels.unlock_aku_realm.unlock_aku_realm,
        "Gauntlets": levels.gauntlet.edit_gauntlet,
        "Collab Gauntlets": levels.gauntlet.edit_collab_gauntlet,
        "Towers": levels.towers.edit_tower,
        "Behemoth Culling": levels.behemoth_culling.edit_behemoth_culling,
        "Into the Future Timed Scores": levels.itf_timed_scores.timed_scores,
        "Challenge Battle Score": basic.basic_items.edit_challenge_battle,
        "Clear Tutorial": levels.clear_tutorial.clear_tutorial,
        "Catclaw Dojo Score (Hall of Initiates)": basic.basic_items.edit_dojo_score,
        "Add Enigma Stages": levels.enigma_stages.edit_enigma_stages,
        "Allow the filibuster stage to be recleared": levels.allow_filibuster_clearing.allow_filibuster_clearing,
        "Legend Quest": levels.legend_quest.edit_legend_quest,
    },
    "Inquiry Code / Token / Account": {
        "Inquiry Code": basic.basic_items.edit_inquiry_code,
        "Token": basic.basic_items.edit_token,
        "Fix elsewhere error / Unban account": other.fix_elsewhere.fix_elsewhere,
        "Old Fix elsewhere error / Unban account (needs 2 save files)": fix_elsewhere_old,
        "Generate a new inquiry code and token": other.create_new_account.create_new_account,
    },
    "Other": {
        "Rare Gacha Seed": basic.basic_items.edit_rare_gacha_seed,
        "Unlocked Equip Slots": basic.basic_items.edit_unlocked_slots,
        "Get Restart Pack / Returner Mode": basic.basic_items.edit_restart_pack,
        "Meow Medals": other.meow_medals.medals,
        "Play Time": other.play_time.edit_play_time,
        "Unlock / Remove Enemy Guide Entries": other.unlock_enemy_guide.enemy_guide,
        "Catnip Challenges / Missions": other.missions.edit_missions,
        "Normal Ticket Max Trade Progress (allows for unbannable rare tickets)": other.trade_progress.set_trade_progress,
        "Get / Remove Gold Pass": other.get_gold_pass.get_gold_pass,
        "Claim / Remove all user rank rewards (does not give any items)": other.claim_user_rank_rewards.edit_rewards,
        "Cat Shrine Level / XP": other.cat_shrine.edit_shrine_xp,
    },
    "Fixes": {
        "Fix time errors": other.fix_time_issues.fix_time_issues,
        "Unlock the Equip Menu": other.unlock_equip_menu.unlock_equip,
        "Clear Tutorial": levels.clear_tutorial.clear_tutorial,
        "Fix elsewhere error / Unban account": other.fix_elsewhere.fix_elsewhere,
        "Old Fix elsewhere error / Unban account (needs 2 save files)": fix_elsewhere_old,
        "Fix gamatoto from crashing the game": gamototo.fix_gamatoto.fix_gamatoto,
    },
    "Edit Config": {
        "Edit LOCALIZATION": config_manager.edit_locale,
        "Edit DEFAULT_COUNTRY_CODE": config_manager.edit_default_gv,
        "Edit DEFAULT_SAVE_PATH": config_manager.edit_default_save_file_path,
        "Edit FIXED_SAVE_PATH": config_manager.edit_fixed_save_path,
        "Edit EDITOR settings": config_manager.edit_editor_settings,
        "Edit START_UP settings": config_manager.edit_start_up_settings,
        "Edit SAVE_CHANGES settings": config_manager.edit_save_changes_settings,
        "Edit SERVER settings": config_manager.edit_server_settings,
        "Edit config path": config_manager.edit_config_path,
    },
    "Exit": helper.exit_check_changes,
}


def get_feature(
    selected_features: Any, search_string: str, results: dict[str, Any]
) -> dict[str, Any]:
    """Search for a feature if the feature name contains the search string"""

    for feature in selected_features:
        feature_data = selected_features[feature]
        if isinstance(feature_data, dict):
            feature_data = get_feature(feature_data, search_string, results)
        if search_string.lower().replace(" ", "") in feature.lower().replace(" ", ""):
            results[feature] = selected_features[feature]
    return results


def show_options(
    save_stats: dict[str, Any], features_to_use: dict[str, Any]
) -> dict[str, Any]:
    """Allow the user to either enter a feature number or a feature name, and get the features that match"""

    if (
        not config_manager.get_config_value_category("EDITOR", "SHOW_CATEGORIES")
        and FEATURES == features_to_use
    ):
        user_input = ""
    else:
        prompt = (
            "옵션 선택"
        )
 
        user_input = user_input_handler.colored_input(f"{prompt}:\n")
    user_int = helper.check_int(user_input)
    results = []
    if user_int is None:
        results = get_feature(features_to_use, user_input, {})
    else:
        if user_int < 1 or user_int > len(features_to_use) + 1:
            helper.colored_text("Value out of range", helper.RED)
            return show_options(save_stats, features_to_use)
        if FEATURES != features_to_use:
            if user_int - 2 < 0:
                return menu(save_stats)
            results = features_to_use[list(features_to_use)[user_int - 2]]
        else:
            results = features_to_use[list(features_to_use)[user_int - 1]]
    if not isinstance(results, dict):
        save_stats_return = results(save_stats)
        if save_stats_return is None:
            return save_stats
        return save_stats_return
    if len(results) == 0:
        helper.colored_text("No feature found with that name.", helper.RED)
        return menu(save_stats)
    if len(results) == 1 and isinstance(list(results.values())[0], dict):
        results = results[list(results)[0]]
    if len(results) == 1:
        save_stats_return = results[list(results)[0]](save_stats)
        if save_stats_return is None:
            return save_stats
        return save_stats_return

    helper.colored_list(["Go Back"] + list(results))
    return show_options(save_stats, results)


def menu(
    save_stats: dict[str, Any], path_save: Union[str, None] = None
) -> dict[str, Any]:
    """Show the menu and allow the user to select a feature to edit"""

    if path_save:
        helper.set_save_path(path_save)
    if config_manager.get_config_value_category("EDITOR", "SHOW_CATEGORIES"):
        helper.colored_list(list(FEATURES))
    save_stats = show_options(save_stats, FEATURES)

    return save_stats



"""Handler for fixing time issues"""
from typing import Any
from ... import helper


def fix_time_issues(save_stats: dict[str, Any]) -> dict[str, Any]:
    """
    Fix time issues

    Args:
        save_stats (dict[str, Any]): Save stats

    Returns:
        dict[str, Any]: Save stats
    """
    save_stats["third_time"] = helper.get_iso_time()

    save_stats["time_stamp"] = helper.get_time()
    save_stats["time_stamp_4"] = helper.get_time()

    helper.colored_text(
        "Successfully fixed time issues &(Your device time on both devices must be correct for this to work!)&",
        helper.GREEN,
        helper.RED,
    )
    return save_stats
